@@ -2,17 +2,16 @@
 
 /* This function closes the audio device. */
 
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <unistd.h>
 
 #include "rawrec.h"
 
-void audio_close(int audio_fd, char *audio_dev)
+void audio_close(int audio_fd, const char *audio_dev)
 { 
-  if ( close(audio_fd) == -1 ) {
-    fprintf(stderr, "%s: could not close %s: ", progname, audio_dev);
-    perror(""); 
-    exit(EXIT_FAILURE);
-  }
+  if ( close(audio_fd) == -1 )
+    err_die("could not close %s: %s\n", audio_dev, strerror(errno));
 }
