@@ -27,6 +27,6 @@ int get_format_code(const char *format)
     return AFMT_U8;
   
   /* If we make it this far, its a bug.  */
-  fprintf(stderr, "%s: BUG: internal function get_format_code got invalid string argument\n", progname);
-  exit(EXIT_FAILURE);
+  err_die("BUG: internal function '%s' got invalid string argument\n", 
+	  __func__);
 }
