@@ -3,6 +3,7 @@
 /* This function performs the actual playing run as specified by the
    options. */
 
+#include <errno.h>
 #include <math.h>
 #include <sched.h>
 #include <stdio.h>
@@ -25,7 +26,7 @@
    can't POSIXly make the pthread calls directly from the handler and
    need to work around some linux pthread signal handling deficiency.  */
 extern int got_watched_for_shutdown_signal;
-void shutdown_signal_handler(void);
+void shutdown_signal_handler(int signum);
 /* This allows the following thread (move_au for play, move_fd for
    record) to tell this main thread when they finish. */
 pthread_mutex_t tell_main_follower_done_mutex = PTHREAD_MUTEX_INITIALIZER;
@@ -113,8 +114,8 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
                (strcmp(clp->format, "u8") == 0) )
     bps = 8;
   else {
-    fprintf(stderr, "%s: BUG: unrecognized sample format seen in function record\n", progname);
-    exit(EXIT_FAILURE);
+    err_die("BUG: unrecognized sample format seen in function '%s'\n", 
+	    __func__);
   }
 
   /* Open the argument data file to be read from, or use standard input. */
@@ -127,10 +128,8 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
   if (app->using_stdio == FALSE )
     /* stat the argument file.  we're interested in it's size in bytes. */
     if ( fstat(arg_fd, &arg_file_stats) == -1 ) {
-      fprintf(stderr, "%s: fstat of argument file %s failed: ", progname,
-               app->arg_file);
-      perror("");
-      exit(EXIT_FAILURE);
+      err_die("fstat of argument file %s failed: %s\n", app->arg_file, 
+	      strerror(errno));
     }
 
   /* Determine the actual amount of data to be played, not including
@@ -159,13 +158,16 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
   /* Compute the number of bytes to jump into the file.  If both -j
      seconds and -J samples are specified, the one which is longest in
      real time (farthest into the file) in the one used. */
-  if ( app->time_startjump != 0 || app->samp_startjump != 0 ) {
+  if ( app->time_startjump > 0 || app->samp_startjump > 0 ) {
     if ( app->time_startjump > ( (double) app->samp_startjump) 
                / app->speed ) {
       jump_bytes = floor( (double) app->time_startjump 
 			  * (double) app->speed * (double) bps 
 			  * (double) (app->channels) / 8.0);
       /* We must be sure to jump a multiple of (app->channels * bps / 8). */
+      /* FIXME: The use of doubles in this loop conditional causes a
+	 compiler warning, which is ok since doubles need to stop
+	 being used for this sort of stuff.  */
       for ( ; jump_bytes / (app->channels * bps / 8) 
 	      != floor(jump_bytes / (app->channels * bps / 8)) 
 	      ; jump_bytes-- )
@@ -185,19 +187,15 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
          situations.  */
       if ( (tmp_buf = (unsigned char *) malloc( (size_t) audio_fragsz)) 
                == NULL ) {
-	fprintf(stderr, "%s: malloc failed: ", progname);
-	perror("");
-	exit(EXIT_FAILURE);
+	err_die("malloc failed: %s\n", strerror(errno));
       }
 
       /* Do the reads (and throw away the results).  */
       for ( bytes_jumped = 0 ; bytes_jumped < jump_bytes ; 
                bytes_jumped += bytes_read ) {
-        if ( (bytes_read = read(arg_fd, tmp_buf, min(audio_fragsz, 
+        if ( (bytes_read = read(arg_fd, tmp_buf, (size_t) min(audio_fragsz, 
                jump_bytes - bytes_jumped))) == -1 ) {
-	  fprintf(stderr, "%s: read from standard input failed: ", progname);
-	  perror("");
-	  exit(EXIT_FAILURE);
+	  err_die("read from standard input failed: %s\n", strerror(errno));
 	}
 
         /* Count empty reads in a row.  */
@@ -212,12 +210,11 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
              use it.  It needs to go away eventually or something.  */
           if ( (app->samp_limit_set == TRUE) 
                || (app->time_limit_set == TRUE) ) {
-	    fprintf(stderr, "%s: too many empty segments seen while skipping as per -j or -J options (starved for standard input), giving up\n", progname);
-	    exit(EXIT_FAILURE);
+	    err_die("too many empty segments seen while skipping as per -j or -J options (starved for standard input), giving up\n");
 	  } else { /* th_arg->limit_set == FALSE */
-	    if ( app->verbose == TRUE )
-	      fprintf(stderr, "%s: warning: standard input ended without supplying enough data to satisfy the given skip option (-j or -J)\n", progname);
-	    exit(EXIT_SUCCESS);
+	    if ( app->verbose == TRUE ) {
+	      err_die("warning: standard input ended without supplying enough data to satisfy the given skip option (-j or -J)\n");
+	    }
 	  }
 	}
       }
@@ -227,10 +224,8 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
     } else { /* app->using_stdio == FALSE */
       /* Use lseek to do the actual jumping.  */
       if ( lseek(arg_fd, (off_t) jump_bytes, SEEK_SET) == -1 ) {
-	fprintf(stderr, "%s: lseek on %s failed: ", progname, app->arg_file);
-	perror("");
-	exit(EXIT_FAILURE);
-      } 
+	err_die("lseek on %s failed: %s\n", app->arg_file, strerror(errno));
+      }
     }
   }
 
@@ -262,114 +257,73 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
   /* Set thread attributes.  See thread_scheme.txt in docs/programmer
      for rational. */
   if ( (rtn = pthread_attr_init(&move_au_attr)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_init failed: %s\n", progname, 
-	    strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_attr_init failed: %s\n", strerror(rtn));
   }
   if ( (rtn = pthread_attr_setdetachstate(&move_au_attr, 
 					  PTHREAD_CREATE_JOINABLE)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_setdetachstate failed: %s\n",
-               progname, strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_attr_setdetachstate failed: %s\n", strerror(rtn));
   }
-#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
+#if defined (_POSIX_THREAD_PRIORITY_SCHEDULING) \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != -1 \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != 0
   if ( have_root_authority ) {
     if ( (rtn = pthread_attr_setinheritsched(&move_au_attr, 
 					     PTHREAD_EXPLICIT_SCHED)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setinheritsched failed: %s\n",
-               progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setschedpolicy(&move_au_attr, SCHED_FIFO)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedpolicy failed: %s\n", 
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (fifo_max_prio = sched_get_priority_max(SCHED_FIFO)) == -1 ) {
-      fprintf(stderr, "%s: BUG: sched_get_priority_max failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
+      err_die("BUG: pthread_attr_setinheritsched failed: %s\n", strerror(rtn));
     }
+    if ( (rtn = pthread_attr_setschedpolicy(&move_au_attr, SCHED_FIFO)) )
+      err_die("BUG: pthread_attr_setschedpolicy failed: %s\n", strerror(rtn));
+    if ( (fifo_max_prio = sched_get_priority_max(SCHED_FIFO)) == -1 )
+      err_die("BUG: sched_get_priority_max failed: %s", strerror(errno));
     move_au_param.sched_priority = fifo_max_prio;
-    if ( (rtn = pthread_attr_setschedparam(&move_au_attr, &move_au_param)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedparam failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setscope(&move_au_attr, PTHREAD_SCOPE_SYSTEM)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setscope failed: %s\n", progname,
-	       strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
+    if ( (rtn = pthread_attr_setschedparam(&move_au_attr, &move_au_param)) )
+      err_die("BUG: pthread_attr_setschedparam failed: %s\n", strerror(rtn));
+    if ( (rtn = pthread_attr_setscope(&move_au_attr, PTHREAD_SCOPE_SYSTEM)) )
+      err_die("BUG: pthread_attr_setscope failed: %s\n", strerror(rtn));
   }
 #endif
-  if ( (rtn = pthread_attr_init(&move_fd_attr)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_init failed: %s\n", progname,
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
+  if ( (rtn = pthread_attr_init(&move_fd_attr)) )
+    err_die("BUG: pthread_attr_init failed: %s\n", strerror(rtn));
   if ( (rtn = pthread_attr_setdetachstate(&move_fd_attr, 
 					  PTHREAD_CREATE_JOINABLE)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_setdetachstate failed: %s\n",
-	       progname, strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_attr_setdetachstate failed: %s\n", strerror(rtn));
   }
-#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
+
+#if defined (_POSIX_THREAD_PRIORITY_SCHEDULING) \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != -1 \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != 0
   if ( have_root_authority ) {
     if ( (rtn = pthread_attr_setinheritsched(&move_fd_attr, 
 					     PTHREAD_EXPLICIT_SCHED)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setinheritsched failed: %s\n",
-               progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setschedpolicy(&move_fd_attr, SCHED_FIFO)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedpolicy failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
+      err_die("BUG: pthread_attr_setinheritsched failed: %s\n", strerror(rtn));
     }
+    if ( (rtn = pthread_attr_setschedpolicy(&move_fd_attr, SCHED_FIFO)) )
+      err_die("BUG: pthread_attr_setschedpolicy failed: %s\n", strerror(rtn));
     move_fd_param.sched_priority = fifo_max_prio;
-    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_au_param)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedparam failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setscope(&move_fd_attr, PTHREAD_SCOPE_SYSTEM)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setscope failed: %s\n", 
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
+    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_au_param)) )
+      err_die("BUG: pthread_attr_setschedparam failed: %s\n", strerror(rtn));
+    if ( (rtn = pthread_attr_setscope(&move_fd_attr, PTHREAD_SCOPE_SYSTEM)) )
+      err_die("BUG: pthread_attr_setscope failed: %s\n", strerror(rtn));
   }
 #endif
 
   /* Getting ugly.  Here we install a handler (which sets a global
      flag which the threads can poll in order to do graceful
      death). */
-  shutdown_handler_act.sa_handler = (void (*)(int)) shutdown_signal_handler;
+  shutdown_handler_act.sa_handler = shutdown_signal_handler;
   sigfillset(&(shutdown_handler_act.sa_mask));
   shutdown_handler_act.sa_flags = 0;
-  if ( sigaction(SIGTERM, &shutdown_handler_act, NULL) == -1 ) { 
-    fprintf(stderr, "%s: sigaction failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
-  if ( sigaction(SIGINT, &shutdown_handler_act, NULL) == -1 ) {
-    fprintf(stderr, "%s: sigaction failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigaction(SIGTERM, &shutdown_handler_act, NULL) == -1 )
+    err_die("sigaction failed: %s\n", strerror(errno));
+  if ( sigaction(SIGINT, &shutdown_handler_act, NULL) == -1 )
+    err_die("sigaction failed: %s\n", strerror(errno));
 
   /* Set up an empty signal set.  */
-  if ( sigfillset(&all_sigs) == -1 ) {
-    fprintf(stderr, "%s: sigfillset failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigfillset(&all_sigs) == -1 )
+    err_die("sigfillset failed: %s\n", strerror(errno));
   /* POSIX requires all signal sets to be initialized before use.  */
-  if ( sigemptyset(&tmp_mask) == -1 ) {
-    fprintf(stderr, "%s: sigemptyset failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigemptyset(&tmp_mask) == -1 )
+    err_die("sigemptyset failed: %s\n", strerror(errno));
   /* Block all signals in preperation for starting threads.  We are
      moving into some linux specific hackery now, though it shouldn't
      make problems elsewhere.  The threads never unblock the signals,
@@ -385,27 +339,18 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
      we don't try to finish playing all buffered data move_fd may have
      loaded for use.  Note that large segment sizes may still result
      in significant signal response latency.  */
-  if ( sigprocmask(SIG_BLOCK, &all_sigs, &tmp_mask) == -1 ) {
-    fprintf(stderr, "%s: sigprocmask failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigprocmask(SIG_BLOCK, &all_sigs, &tmp_mask) == -1 )
+    err_die("sigprocmask failed: %s\n", strerror(errno));
 
   if ( have_root_authority ) {
     /* Get root authority (the saved set-user id should be root).  */
-    if ( seteuid( (uid_t) 0 ) == -1 ) {
-      fprintf(stderr, "%s: seteuid( (uid_t) 0 ) failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( seteuid( (uid_t) 0 ) == -1 )
+      err_die("seteuid( (uid_t) 0 ) failed: %s\n", strerror(errno));
 
     /* Entering critical section.  Lock down our memory, if possible.  */
-#ifdef _POSIX_MEMLOCK
-    if ( mlockall(MCL_CURRENT) == -1 ) {
-      fprintf(stderr, "%s: mlockall(MCL_CURRENT) failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+#if defined (_POSIX_MEMLOCK) && _POSIX_MEMLOCK != -1 && _POSIX_MEMLOCK != 0
+    if ( mlockall(MCL_CURRENT) == -1 )
+      err_die("mlockall(MCL_CURRENT) failed: %s\n", strerror(errno));
 #endif
   }
 
@@ -415,23 +360,20 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
      ids, but drop them immediately on entry. */  
   if ( (rtn = pthread_create(&move_au_th, &move_au_attr, 
 			     (void *(*)(void *)) move_au, &au_th_arg)) ) {
-    fprintf(stderr, "%s: BUG: pthread_create failed: %s\n", progname,
-               strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
   }
   if ( (rtn = pthread_create(&move_fd_th, &move_fd_attr, 
 			     (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
-    fprintf(stderr, "%s: BUG: pthread_create failed: %s\n", progname, 
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
   }
 
-  if ( have_root_authority )
-    /* Drop back to normal uid. */
+  if ( have_root_authority ) {
+    /* Drop back to normal uid authority. */
     if ( seteuid(getuid()) == -1 )
       /* If for some crazy reason we fail to drop root permissions, exit 
 	 immediately without advertising the fact. */
-      exit(EXIT_FAILURE);
+      err_die("BUG: internal error\n");
+  }
 
   /* Restore the default mask.  Note that at no time has the default
      action for the shutdown signals we handle (sloppy death in which
@@ -439,11 +381,8 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
      thrash on for a brief period, hopefully this mess will be fixed
      with kernel 2.5 and glibc 2.3) been allowed to take place after
      threads have been spawned.  */
-  if ( sigprocmask(SIG_SETMASK, &tmp_mask, NULL) == -1 ) {
-    fprintf(stderr, "%s: sigprocmask failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigprocmask(SIG_SETMASK, &tmp_mask, NULL) == -1 )
+    err_die("sigprocmask failed: %s\n", strerror(errno));
 
   /* Now here is some sad hackery.  We can't sigwait() because these
      are asynchronous signals we are worried about, and they might
@@ -461,7 +400,7 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
      it doesn't find any data to read or finishes normally.  It has to
      let this main thread know when this happens so we can stop
      sleeping and waiting for signals and commence pthread_joining.
-     It tells us via mutex-protected tell_main_end_stdin, which we
+     It tells us via mutex-protected tell_main_follower_done, which we
      poll from here every half audio fragsz worth of time (hopefully
      giving maximum latency < fragsz as advertized).  */
   sleep_time_secs = (8.0 * audio_fragsz) / (2 * bps 
@@ -486,21 +425,12 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
        an int should be pretty darn atomic, and if we have made it this
        far without getting the signal we no longer care much whether we
        get it or not.  */
-    if ( sigemptyset(&tmp_mask) == -1 ) {
-      fprintf(stderr, "%s: sigemptyset failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
-    if ( sigaddset(&tmp_mask, SIGTERM) == -1 ) {
-      fprintf(stderr, "%s: sigaddset failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
-    if ( sigprocmask(SIG_BLOCK, &tmp_mask, NULL) == -1 ) {
-      fprintf(stderr, "%s: sigprocmask failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( sigemptyset(&tmp_mask) == -1 )
+      err_die("sigemptyset failed: %s\n", strerror(errno));
+    if ( sigaddset(&tmp_mask, SIGTERM) == -1 )
+      err_die("sigaddset failed: %s\n", strerror(errno));
+    if ( sigprocmask(SIG_BLOCK, &tmp_mask, NULL) == -1 )
+      err_die("sigprocmask failed: %s\n", strerror(errno));
     if ( got_watched_for_shutdown_signal ) {
       /* Let the move_au and move_fd threads know that we have been
 	 interrupted by a signal that we catch in order to do clean
@@ -513,41 +443,23 @@ void play(parameters_stt *clp) /* pneumonic: app: actual params pointer */
       pthread_mutex_lock(&shutdown_signal_seen_mutex);
       shutdown_signal_seen = 1;
       pthread_mutex_unlock(&shutdown_signal_seen_mutex);
-      if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 ) {
-	fprintf(stderr, "%s: sigprocmask failed: ", progname);
-	perror("");
-	exit(EXIT_FAILURE);
-      }
+      if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 )
+	err_die("sigprocmask failed: %s\n", strerror(errno));
       break;
     }
-    if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 ) {
-      fprintf(stderr, "%s: sigprocmask failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 )
+      err_die("sigprocmask failed: %s\n", strerror(errno));
   }
   
   /* Wait for threads to finish. */
-  if ( (rtn = pthread_join(move_au_th, &au_th_ret)) ) {
-    fprintf(stderr, "%s: BUG: pthread_join failed: %s\n", progname,
-               strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
-  if ( *( (int *) au_th_ret) == -1 ) {
-    fprintf(stderr, "%s: abnormal termination of move_au_th, aborting.\n",
-	       progname);
-    exit(EXIT_FAILURE);
-  }
-  if ( (rtn = pthread_join(move_fd_th, &fd_th_ret)) ) {
-    fprintf(stderr, "%s: BUG: pthread_join failed: %s\n", progname,
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
-  if ( *( (int *) fd_th_ret) == -1 ) {
-    fprintf(stderr, "%s: abnormal termination of move_fd_th, aborting.\n",
-	       progname);
-    exit(EXIT_FAILURE);
-  }
+  if ( (rtn = pthread_join(move_au_th, &au_th_ret)) )
+    err_die("BUG: pthread_join failed: %s\n", strerror(rtn));
+  if ( *( (int *) au_th_ret) == -1 )
+    err_die("abnormal termination of move_au_th, aborting\n");
+  if ( (rtn = pthread_join(move_fd_th, &fd_th_ret)) )
+    err_die("BUG: pthread_join failed: %s\n", strerror(rtn));
+  if ( *( (int *) fd_th_ret) == -1 )
+    err_die("abnormal termination of move_fd_th, aborting\n");
 
   /* As per our promise not to do things like end record (-e or -E) or
      end pause (-z or -Z) if we got interrupted by a signal.  This
