@@ -3,6 +3,10 @@
 /* This function performs the actual recording run as specified by the
    options. */
 
+#include <assert.h>/***/
+#include <stdio.h>/***/
+
+#include <errno.h>
 #include <math.h>
 #include <sched.h>
 #include <stdio.h>
@@ -22,7 +26,7 @@
    since we can't POSIXly make the pthread calls directly from the
    handler.  */
 extern int got_watched_for_shutdown_signal;
-void shutdown_signal_handler(void);
+void shutdown_signal_handler(int signum);
 /* This allows the following thread (move_au for play, move_fd for
    record) to tell this main thread when they finish. */
 extern pthread_mutex_t tell_main_follower_done_mutex;
@@ -30,31 +34,30 @@ extern int tell_main_follower_done;
 
 void record(parameters_stt *clp) 
 {
-  int audio_fd = 0;	  /* Audio device file descriptor. */
-  int arg_fd;		  /* Argument file fd. */
-  int bps;		  /* the bits per sample for the format in use */
-  double rec_bytes;	  /* bytes to record, not counting silent padding */
+  int audio_fd = 0;	  /* Audio device file descriptor.  */
+  int arg_fd;		  /* Argument file fd.  */
+  int bps;		  /* Bits per sample for the format in use.  */
+  double rec_bytes;	  /* Bytes to record, not counting silent padding.  */
   long audio_fragsz;	  /* Kernel audio fragment size. */
   int ringbuf_segs;	  /* Number of segments in ringbuf. */
-  move_au_th_arg_stt au_th_arg;	  /* move_au_th argument structure. */
-  move_fd_th_arg_stt fd_th_arg;	  /* move_fd_th argument structure. */
+  move_au_th_arg_stt au_th_arg;	  /* Thread argument structure.  */
+  move_fd_th_arg_stt fd_th_arg;	  /* Thread argument structure.  */
   int rtn;		  /* For return values of pthread fctns. */
-  /* Maximum priority of FIFO thread.  Should always be initialized
+  /* Maximum priority of FIFO thread.  Should always be rinitialized
      elsewhere before use.  */
   int fifo_max_prio = 0;	  
   void *au_th_ret;   	  /* Audio thread return pointer. */
   void *fd_th_ret;	  /* File thread return pointer. */
   sigset_t all_sigs;      /* Full set of all signals.  */
   sigset_t tmp_mask;	  /* Temporary signal set storage.  */
-  /* sigaction structure for shutdown handler */
-  struct sigaction shutdown_handler_act;  
+  struct sigaction shutdown_handler_act;   /* For shutdown handler.  */
   double sleep_time_secs;	  /* Compute for nanosleep().  */
   struct timespec sleep_time;	  /* For nanosleep() call.  */
   /* The actual parameters used for the record/play run may differ slightly
      from those given on the command line due to hardware limitations. */
   parameters_stt actual_params;
-  /* pointer to the actual_params structure */
-  parameters_stt *app = NULL;	/* bogus initializer reasures compiler */
+  /* Pointer to the actual_params structure.  */
+  parameters_stt *app = NULL;	  /* bogus initializer reasures compiler */
 
   /* If the user wants us to selfishly hold the audio device, grab it now. */
   if ( clp->hold_audio_device == TRUE ) {
@@ -105,11 +108,9 @@ void record(parameters_stt *clp)
   else if ( (strcmp(clp->format, "s8") == 0) 
 	       || (strcmp(clp->format, "u8") == 0) )
     bps = 8;
-  else {
-    fprintf(stderr, "%s: BUG: unrecognized sample format seen in function record\n", progname);
-    exit(EXIT_FAILURE);
-  }
-    
+  else
+    err_die("BUG: unrecognized sample format seen in function '%s'\n",
+	    __func__);
 
   /* Open the argument data file to be written to, or use standard output. */
   if ( app->using_stdio == TRUE )
@@ -178,115 +179,74 @@ void record(parameters_stt *clp)
 
   /* Set thread attributes.  See thread_scheme.txt in docs/programmer
      for rational. */
-  if ( (rtn = pthread_attr_init(&move_au_attr)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_init failed: %s\n", progname, 
-               strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
+  if ( (rtn = pthread_attr_init(&move_au_attr)) )
+    err_die("BUG: pthread_attr_init failed: %s\n", strerror(rtn));
   if ( (rtn = pthread_attr_setdetachstate(&move_au_attr, 
 					  PTHREAD_CREATE_JOINABLE)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_setdetachstate failed: %s\n", 
-               progname, strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_attr_setdetachstate failed: %s\n", strerror(rtn));
   }
-#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
+#if defined (_POSIX_THREAD_PRIORITY_SCHEDULING) \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != -1 \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != 0
   if ( have_root_authority ) {
     if ( (rtn = pthread_attr_setinheritsched(&move_au_attr, 
 					     PTHREAD_EXPLICIT_SCHED)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setinheritsched failed: %s\n",
-               progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setschedpolicy(&move_au_attr, SCHED_FIFO)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedpolicy failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (fifo_max_prio = sched_get_priority_max(SCHED_FIFO)) == -1 ) {
-      fprintf(stderr, "%s: BUG: sched_get_priority_max failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
+      err_die("BUG: pthread_attr_setinheritsched failed: %s\n", strerror(rtn));
     }
+    if ( (rtn = pthread_attr_setschedpolicy(&move_au_attr, SCHED_FIFO)) )
+      err_die("BUG: pthread_attr_setschedpolicy failed: %s\n", strerror(rtn));
+    if ( (fifo_max_prio = sched_get_priority_max(SCHED_FIFO)) == -1 )
+      err_die("BUG: sched_get_priority_max failed: %s\n", strerror(errno));
     move_au_param.sched_priority = fifo_max_prio;
-    if ( (rtn = pthread_attr_setschedparam(&move_au_attr, &move_au_param)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedparam failed: %s\n", 
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setscope(&move_au_attr, PTHREAD_SCOPE_SYSTEM)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setscope failed: %s\n", progname,
-               strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
+    if ( (rtn = pthread_attr_setschedparam(&move_au_attr, &move_au_param)) )
+      err_die("BUG: pthread_attr_setschedparam failed: %s\n", strerror(rtn));
+    if ( (rtn = pthread_attr_setscope(&move_au_attr, PTHREAD_SCOPE_SYSTEM)) )
+      err_die("BUG: pthread_attr_setscope failed: %s\n", strerror(rtn));
   }
 #endif
-  if ( (rtn = pthread_attr_init(&move_fd_attr)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_init failed: %s\n", progname, 
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
+  if ( (rtn = pthread_attr_init(&move_fd_attr)) )
+    err_die("BUG: pthread_attr_init failed: %s\n", strerror(rtn));
   if ( (rtn = pthread_attr_setdetachstate(&move_fd_attr, 
 					  PTHREAD_CREATE_JOINABLE)) ) {
-    fprintf(stderr, "%s: BUG: pthread_attr_setdetachstate failed: %s\n",
-	       progname, strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_attr_setdetachstate failed: %s\n", strerror(rtn));
   }
-#ifdef _POSIX_THREAD_PRIORITY_SCHEDULING
+#if defined (_POSIX_THREAD_PRIORITY_SCHEDULING) \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != -1 \
+    && _POSIX_THREAD_PRIORITY_SCHEDULING != 0
   if ( have_root_authority ) {
     if ( (rtn = pthread_attr_setinheritsched(&move_fd_attr, 
 					     PTHREAD_EXPLICIT_SCHED)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setinheritsched failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setschedpolicy(&move_fd_attr, SCHED_FIFO)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedpolicy failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
+      err_die("BUG: pthread_attr_setinheritsched failed: %s\n", strerror(rtn));
     }
+    if ( (rtn = pthread_attr_setschedpolicy(&move_fd_attr, SCHED_FIFO)) )
+      err_die("BUG: pthread_attr_setschedpolicy failed: %s\n", strerror(rtn));
     move_fd_param.sched_priority = fifo_max_prio;
-    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_fd_param)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setschedparam failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
-    if ( (rtn = pthread_attr_setscope(&move_fd_attr, PTHREAD_SCOPE_SYSTEM)) ) {
-      fprintf(stderr, "%s: BUG: pthread_attr_setscope failed: %s\n",
-	       progname, strerror(rtn));
-      exit(EXIT_FAILURE);
-    }
+    if ( (rtn = pthread_attr_setschedparam(&move_fd_attr, &move_fd_param)) )
+      err_die("BUG: pthread_attr_setschedparam failed: %s\n", strerror(rtn));
+    if ( (rtn = pthread_attr_setscope(&move_fd_attr, PTHREAD_SCOPE_SYSTEM)) )
+      err_die("BUG: pthread_attr_setscope failed: %s\n", strerror(rtn));
   }
 #endif
 
   /* Getting ugly.  Here we install a handler (which sets a global
      flag which the threads can poll in order to do graceful
      death). */
-  shutdown_handler_act.sa_handler = (void (*)(int)) shutdown_signal_handler;
+  shutdown_handler_act.sa_handler = shutdown_signal_handler;
   sigfillset(&(shutdown_handler_act.sa_mask));
   shutdown_handler_act.sa_flags = 0;
-  if ( sigaction(SIGTERM, &shutdown_handler_act, NULL) == -1 ) { 
-    fprintf(stderr, "%s: sigaction failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
-  if ( sigaction(SIGINT, &shutdown_handler_act, NULL) == -1 ) {
-    fprintf(stderr, "%s: sigaction failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigaction(SIGTERM, &shutdown_handler_act, NULL) == -1 )
+    err_die("sigaction failed: %s\n", strerror(errno));
+  if ( sigaction(SIGINT, &shutdown_handler_act, NULL) == -1 )
+    err_die("sigaction failed: %s\n", strerror(errno));
 
   /* Set up an empty signal set.  */
-  if ( sigfillset(&all_sigs) == -1 ) {
-    fprintf(stderr, "%s: sigfillset failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigfillset(&all_sigs) == -1 )
+    err_die("sigfillset failed: %s\n", strerror(errno));
+
   /* POSIX requires all signal sets to be initialized before use.  */
-  if ( sigemptyset(&tmp_mask) == -1 ) {
-    fprintf(stderr, "%s: sigemptyset failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigemptyset(&tmp_mask) == -1 )
+    err_die("sigemptyset failed: %s\n", strerror(errno));
+
   /* Block all signals in preperation for starting threads.  We are
      moving into some linux specific hackery now, though it shouldn't
      make problems elsewhere.  The threads never unblock the signals,
@@ -302,27 +262,18 @@ void record(parameters_stt *clp)
      we don't try to finish playing all buffered data move_fd may have
      loaded for use.  Note that large segment sizes may still result
      in significant signal response latency.  */
-  if ( sigprocmask(SIG_BLOCK, &all_sigs, &tmp_mask) == -1 ) {
-    fprintf(stderr, "%s: sigprocmask failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigprocmask(SIG_BLOCK, &all_sigs, &tmp_mask) == -1 )
+    err_die("sigprocmask failed: %s\n", strerror(errno));
 
   if ( have_root_authority ) {
     /* Get root authority (the saved set-user id should be root).  */
-    if ( seteuid( (uid_t) 0 ) == -1 ) {
-      fprintf(stderr, "%s: seteuid( (uid_t) 0 ) failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( seteuid( (uid_t) 0 ) == -1 ) 
+      err_die("seteuid( (uid_t) 0 ) failed: %s\n", strerror(errno));
 
     /* Entering critical section.  Lock down our memory, if possible.  */
-#ifdef _POSIX_MEMLOCK
-    if ( mlockall(MCL_CURRENT) == -1 ) {
-      fprintf(stderr, "%s: mlockall(MCL_CURRENT) failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+#if defined (_POSIX_MEMLOCK) && _POSIX_MEMLOCK != -1 && _POSIX_MEMLOCK != 0
+    if ( mlockall(MCL_CURRENT) == -1 )
+      err_die("mlockall(MCL_CURRENT) failed: %s\n", strerror(errno));
 #endif
   }
 
@@ -332,23 +283,20 @@ void record(parameters_stt *clp)
      ids, but drop them immediately on entry. */  
   if ( (rtn = pthread_create(&move_au_th, &move_au_attr, 
 			     (void *(*)(void *)) move_au, &au_th_arg)) ) {
-    fprintf(stderr, "%s: BUG: pthread_create failed: %s\n", progname, 
-               strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
   }
   if ( (rtn = pthread_create(&move_fd_th, &move_fd_attr, 
 			     (void *(*)(void *)) move_fd, &fd_th_arg)) ) {
-    fprintf(stderr, "%s: BUG: pthread_create failed: %s\n", progname,
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
+    err_die("BUG: pthread_create failed: %s\n", strerror(rtn));
   }
 
-  if ( have_root_authority )
+  if ( have_root_authority ) {
     /* Drop back to real uid authority. */
     if ( seteuid(getuid()) == -1 )
       /* If for some crazy reason we fail to drop root permissions, exit
 	 immediately without advertising the fact. */
-      exit(EXIT_FAILURE);
+      err_die("BUG: internal error\n");
+  }
 
   /* Restore the default mask.  Note that at no time has the default
      action for the shutdown signals we handle (sloppy death in which
@@ -356,11 +304,8 @@ void record(parameters_stt *clp)
      thrash on for a brief period, hopefully this mess will be fixed
      with kernel 2.5 and glibc 2.3) been allowed to take place after
      threads have been spawned.  */
-  if ( sigprocmask(SIG_SETMASK, &tmp_mask, NULL) == -1 ) {
-    fprintf(stderr, "%s: sigprocmask failed: ", progname);
-    perror("");
-    exit(EXIT_FAILURE);
-  }
+  if ( sigprocmask(SIG_SETMASK, &tmp_mask, NULL) == -1 )
+    err_die("sigprocmask failed: %s\n", strerror(errno));
 
   /* Now here is some sad hackery.  We can't sigwait() because these
      are asynchronous signals we are worried about, and they might
@@ -378,7 +323,7 @@ void record(parameters_stt *clp)
      it doesn't find any data to read or finishes normally.  It has to
      let this main thread know when this happens so we can stop
      sleeping and waiting for signals and commence pthread_joining.
-     It tells us via mutex-protected tell_main_end_stdin, which we
+     It tells us via mutex-protected tell_main_follower_done, which we
      poll from here every half audio fragsz worth of time (hopefully
      giving maximum latency < fragsz as advertized).  */
   sleep_time_secs = (8.0 * audio_fragsz) / (2 * bps 
@@ -404,26 +349,14 @@ void record(parameters_stt *clp)
        an int should be pretty darn atomic, and if we have made it this
        far without getting the signal we no longer care much whether we
        get it or not.  */
-    if ( sigemptyset(&tmp_mask) == -1 ) {
-      fprintf(stderr, "%s: sigemptyset failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
-    if ( sigaddset(&tmp_mask, SIGTERM) == -1 ) {
-      fprintf(stderr, "%s: sigaddset failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
-    if ( sigaddset(&tmp_mask, SIGINT) == -1 ) {
-      fprintf(stderr, "%s: sigaddset failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
-    if ( sigprocmask(SIG_BLOCK, &tmp_mask, NULL) == -1 ) {
-      fprintf(stderr, "%s: sigprocmask failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( sigemptyset(&tmp_mask) == -1 )
+      err_die("sigemptyset failed: %s\n", strerror(errno));
+    if ( sigaddset(&tmp_mask, SIGTERM) == -1 )
+      err_die("sigaddset failed: %s\n", strerror(errno));
+    if ( sigaddset(&tmp_mask, SIGINT) == -1 )
+      err_die("sigaddset failed: %s\n", strerror(errno));
+    if ( sigprocmask(SIG_BLOCK, &tmp_mask, NULL) == -1 )
+      err_die("sigprocmask failed: %s\n", strerror(errno));
     if ( got_watched_for_shutdown_signal ) {
       /* Let the move_au and move_fd threads know that we have been
 	 interrupted by a signal that we catch in order to do clean
@@ -436,42 +369,24 @@ void record(parameters_stt *clp)
       pthread_mutex_lock(&shutdown_signal_seen_mutex);
       shutdown_signal_seen = 1;
       pthread_mutex_unlock(&shutdown_signal_seen_mutex);
-      if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 ) {
-	fprintf(stderr, "%s: sigprocmask failed: ", progname);
-	perror("");
-	exit(EXIT_FAILURE);
-      }
+      if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 )
+	err_die("sigprocmask failed: %s\n", strerror(errno));
       break;
     }
-    if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 ) {
-      fprintf(stderr, "%s: sigprocmask failed: ", progname);
-      perror("");
-      exit(EXIT_FAILURE);
-    }
+    if ( sigprocmask(SIG_UNBLOCK, &tmp_mask, NULL) == -1 )
+      err_die("sigprocmask failed: %s\n", strerror(errno));
   }
 
 
   /* Wait for threads to finish. */
-  if ( (rtn = pthread_join(move_au_th, &au_th_ret)) ) {
-    fprintf(stderr, "%s: BUG: pthread_join failed: %s\n", progname, 
-               strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
-  if ( *( (int *) au_th_ret) == -1 ) {
-    fprintf(stderr, "%s: abnormal termination of move_au_th, aborting.\n",
-               progname);
-    exit(EXIT_FAILURE);
-  }
-  if ( (rtn = pthread_join(move_fd_th, &fd_th_ret)) ) {
-    fprintf(stderr, "%s: BUG: pthread_join failed: %s\n", progname, 
-	       strerror(rtn));
-    exit(EXIT_FAILURE);
-  }
-  if ( *( (int *) fd_th_ret) == -1 ) {
-    fprintf(stderr, "%s: abnormal termination of move_fd_th, aborting.\n",
-               progname);
-    exit(EXIT_FAILURE);
-  }
+  if ( (rtn = pthread_join(move_au_th, &au_th_ret)) )
+    err_die("BUG: pthread_join failed: %s\n", strerror(rtn));
+  if ( *( (int *) au_th_ret) == -1 )
+    err_die("abnormal termination of move_au_th, aborting\n");
+  if ( (rtn = pthread_join(move_fd_th, &fd_th_ret)) )
+    err_die("BUG: pthread_join failed: %s\n", strerror(rtn));
+  if ( *( (int *) fd_th_ret) == -1 )
+    err_die("abnormal termination of move_fd_th, aborting\n");
 
   /* As per our promise not to do things like end record (-e or -E) or
      end pause (-z or -Z) if we got interrupted by a signal.  This
